/* property.c
 *
 *  ``pinfocom'' -- a portable Infocom Inc. data file interpreter.
 *  Copyright (C) 1987-1992  InfoTaskForce
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to the
 *  Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Header: RCS/property.c,v 3.0 1992/10/21 16:56:19 pds Stab $
 */

#include    "infocom.h"

extern word  param[];

static property
prop_addr (object_t * obj)
{
    extern byte     *base_ptr;

    property        p;

    p = base_ptr + Z_TO_WORD(obj->data);
    return (p + (Z_TO_BYTE(p) << 1) + 1);
}

static property
next_addr (property p)
{
    return (p + (Z_TO_BYTE(p) >> 5) + 2);
}

static property
search_property (property p, word prop_num)
{
        /* Properties are kept in descending order */

        while ( (*p & 0x1F) > prop_num)
            p = next_addr(p);
	return p;
}

void
next_prop ()
{
    register property    p;
    register word        p_num;
    register word        obj_num=param[0];
    register word        prop_num=param[1];
   
    p = prop_addr(obj_addr(obj_num));
    if (prop_num)
    {
	p = search_property(p,prop_num);
#ifdef 0 
	   if (p_num < prop_num)
            error("prop: Error retrieving next property ($%02x)", prop_num);
        else
#endif		 
            p = next_addr(p);
    }

    store((word)(*p & 0x1F));
}

void
put_prop ()
{
    register property        p;
    register word            p_num;
    register word           obj_num=param[0];
    register word           prop_num=param[1];
    register word           value=param[2];
   

    p = prop_addr(obj_addr(obj_num));
    p = search_property(p,prop_num);
#ifdef 0   
    if (p_num < prop_num)
        error("prop: Error storing property ($%02x)", prop_num);
    else
    {
#endif
        if ((*p++) & 0x20)
        {
            (*p++) = value >> 8;
            *p = value;
        }
        else
            *p = value;
#ifdef 0
    }
#endif
}

void
get_prop ()
{
    register property        p;
    register word            p_num;
    register word            prop;
    register word   obj_num=param[0];
    register word   prop_num=param[1];

    p = prop_addr(obj_addr(obj_num));
    p = search_property(p,prop_num);
    p_num = *p & 0x1F;
    if (p_num < prop_num)
    {
        prop_num = (--prop_num) << 1;
        p = (property) objd.obj_base + prop_num;
        prop = Z_TO_WORD(p);
    }
    else
    {
        if (*(p++) & 0x20)
            prop = Z_TO_WORD(p);
        else
            prop = Z_TO_BYTE(p);
    }
    store(prop);
}

void
get_prop_addr ()
{
    extern byte     *base_ptr;
    register word   obj_num=param[0];
    register word   prop_num=param[1];

    property        p;

    p = prop_addr(obj_addr(obj_num));
    p = search_property(p,prop_num);
    if ( (*p & 0x1F) < prop_num)
        store(0);
    else
        store((word)(p + 1 - base_ptr));
}

void
get_prop_len ()
{
    extern byte     *base_ptr;
    
    property        p;

    p = base_ptr + param[0] - 1;
    store((word)((Z_TO_BYTE(p) >> 5) + 1));
}

void
load_word_array ()
{
    word    page;
    word    page_offset; 
    register word   base;
    register word   offset;

    base=param[0];
    offset=param[1];
    
    base += (offset << 1);
    page = base / BLOCK_SIZE;
    page_offset = base % BLOCK_SIZE;
    store(get_word(page, page_offset));
}

void
load_byte_array ()
{
    word    page;
    word    page_offset;
    register word    base=param[0];
    register word    offset=param[1];

    base += offset;
    page = base / BLOCK_SIZE;
    page_offset = base % BLOCK_SIZE;
    store((word)get_byte(page, page_offset));
}

void
save_word_array ()
{
    extern byte     *base_ptr;
    extern header_t data_head;
    extern byte     *global_ptr; 
   

    register byte    *ptr;
    register word    base=param[0];
    register word    offset=param[1];
    register word    value=param[2];

    /* The quantity added to 'base_ptr' must be a word */

    base += (offset << 1);

    ptr = base_ptr + base;
    (*ptr++) = value >> 8;
    *ptr = value;
}

void
save_byte_array ()
{
    extern byte     *base_ptr;
    extern header_t data_head;
    extern byte     *global_ptr; 

    register byte    *ptr;
    register word    base=param[0];
    register word    offset=param[1];
    register word    value=param[2];

    /* The quantity added to 'base_ptr' must be a word */
    base += offset;

    ptr = base_ptr + base;
    *ptr = value;
}
